/*
 * transfercmd.c - handle transfer commands
 *
 * Written by
 *  Christian Vogelgsang <chris@vogelgsang.org>
 *
 * This file is part of dtv2ser.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "command.h"
#include "display.h"
#include "dtvlow.h"
#include "dtvtrans.h"
#include "uart.h"
#include "uartutil.h"
#include "sertrans.h"
#include "param.h"
#include "timer.h"
#include "joycmd.h"

// ----- Helpers -----

#ifdef USE_LCD
static void lcd_print_dir(u08 to_host)
{
  static u08 *to_host_msg = (u08 *)"d>s";
  static u08 *to_dtv_msg  = (u08 *)"d<s";
  lcd_clear();
  lcd_print_string(0,0,to_host ? to_host_msg : to_dtv_msg);
}
#endif

static void generic_transfer(u08 to_host,
                             host_transfer_funcs_t *host_funcs,
                             dtv_transfer_block_func_t dtv_func)
{
  u08 mode = CMDLINE_ARG_BYTE(0);
  u32 addr = CMDLINE_ARG_DWORD(0);
  u32 len  = CMDLINE_ARG_DWORD(1);

#ifdef USE_LCD
  lcd_print_dir(to_host);
  lcd_print_dword6(8,0,'l',len);
  lcd_print_dword6(8,1,'a',addr);
  lcd_print_byte(0,1,'m',mode);
#endif  

  // perform transfer
  u16 block_size = PARAM_WORD(PARAM_WORD_DTV_TRANSFER_BLOCK_SIZE);
  u08 status = transfer_mem(mode,addr,len,
                            block_size,
                            host_funcs,
                            dtv_func);

#ifdef USE_LCD
  lcd_print_byte(3,0,'s',status);
#endif

  if(status!=TRANSFER_OK) {
    error_condition();
  }
}

#ifdef USE_BLOCKCMD

static void generic_block_transfer(u08 to_host,
                                   host_transfer_funcs_t *host_funcs,
                                   dtv_transfer_block_func_t dtv_func)
{
  u08 mode   = CMDLINE_ARG_BYTE(0);
  u08 bank   = CMDLINE_ARG_BYTE(1);
  u16 offset = CMDLINE_ARG_WORD(0);
  u16 length = CMDLINE_ARG_WORD(1);

#ifdef USE_LCD
  lcd_print_dir(to_host);
  lcd_print_word(10,0,'l',length);
  lcd_print_word(10,1,'o',offset);
  lcd_print_byte(0,1,'m',mode);
  lcd_print_byte(5,1,'b',bank);
#endif  
  
  // perform transfer
  u08 status = transfer_mem_block(mode,bank,offset,length,
                                  host_funcs,
                                  dtv_func);
  
#ifdef USE_LCD
  lcd_print_byte(5,0,'s',status);
#endif
  
  if(status!=TRANSFER_OK) {
    error_condition();
  }
}

#endif

// ---------- Commands ------------------------------------------------------

// ----- Read/Write Memory -----

void exec_read_memory(void)
{
  generic_transfer(1,&serial_host_write_funcs,dtvtrans_recv_mem_block);
}

void exec_write_memory(void)
{
  generic_transfer(0,&serial_host_read_funcs,dtvtrans_send_mem_block);
}

#ifdef USE_DIAGNOSE

void exec_read_memory_only_dtv(void)
{
  diagnose_host_mode = DIAGNOSE_HOST_MODE_WRITE;
  generic_transfer(1,&diagnose_host_transfer_funcs,dtvtrans_recv_mem_block);
}

void exec_write_memory_only_dtv(void)
{
  diagnose_host_mode = DIAGNOSE_HOST_MODE_READ;
  generic_transfer(0,&diagnose_host_transfer_funcs,dtvtrans_send_mem_block);
}

void exec_verify_memory_only_dtv(void)
{
  diagnose_host_mode = DIAGNOSE_HOST_MODE_VERIFY;  
  generic_transfer(1,&diagnose_host_transfer_funcs,dtvtrans_recv_mem_block);
}

#endif // USE_DIAGNOSE

// ----- Serial Diagnose -----

#ifdef USE_DIAGNOSE

void exec_read_memory_only_serial(void)
{
  generic_transfer(1,&serial_host_write_funcs,diagnose_dtv_recv_block);
}

void exec_write_memory_only_serial(void)
{
  generic_transfer(0,&serial_host_read_funcs,diagnose_dtv_send_block);  
}

#endif // USE_DIAGNOSE

// ----- Transfer State -----

void exec_transfer_result(void)
{
  // send result code
  uart_send_hex_byte_crlf(transfer_state.result);
  // send transfer time in 10ms
  uart_send_hex_word_crlf(transfer_state.ms_time);
}

// ----- Read/Write Block Memory -----

#ifdef USE_BLOCKCMD

void exec_block_read_memory(void)
{
  generic_block_transfer(1,&serial_host_write_funcs,dtvtrans_recv_mem_block);
}

void exec_block_write_memory(void)
{
  generic_block_transfer(0,&serial_host_read_funcs,dtvtrans_send_mem_block);
}

#endif

#ifdef USE_DIAGNOSE

#ifdef USE_BLOCKCMD

void exec_block_read_memory_only_dtv(void)
{
  generic_block_transfer(1,&diagnose_host_transfer_funcs,dtvtrans_recv_mem_block);
}

void exec_block_write_memory_only_dtv(void)
{
  generic_block_transfer(0,&diagnose_host_transfer_funcs,dtvtrans_send_mem_block);
}

#endif // USE_BLOCKCMD

#endif // USE_DIAGNOSE
